const typeorm = require('typeorm');
const path = require('path');

module.exports = {
    init: async function (listDBconfig) {
        let DBconfig = listDBconfig;
        DBconfig.database = 'renzheng';
        DBconfig.name = 'renzheng';
        DBconfig.synchronize = true;
        DBconfig.entitySchemas = [path.join(__dirname, 'entity/*.json')];
        this.con = await typeorm.createConnection(DBconfig);

        //不存在此字段  添加
        // if (系统名称) {
        //     let sql = 'desc s_公司表 C_' + 系统名称 + '数据库配置';
        //     let 字段属性 = await this.con.query(sql);
        //     if (字段属性.length < 1) {
        //         await this.con.query('ALTER TABLE s_公司表  ADD COLUMN C_' + 系统名称 + '数据库配置 VARCHAR(500)');
        //     }
        // }
        // if (typeof (新建门店信息.I_上级编号) == 'undefined')
    },
    table: function (tablename) {
        return this.con.getRepository(tablename).createQueryBuilder();
    },
    /*
    注册  添加一条记录     返回用户
    登录  返回用户数据
    创建  返回公司 和门店
    加入门店
    */
    注册: async function (注册数据) {
        if (!注册数据) throw '提交数据错误,data为空!';
        if (!注册数据.手机号) throw '手机号不能为空!';
        if (!注册数据.密码1) throw '密码不能为空!';
        if (!注册数据.密码2) throw '确认密码不能为空!';
        if (注册数据.密码1.length < 6) throw '密码最少6位!';
        if (注册数据.密码1 != 注册数据.密码2) throw '两次输入密码不一样!';

        let regphone = await this.table('s_用户表').where('C_手机号=:zh', { 'zh': 注册数据.手机号 }).getOne();
        if (regphone) throw '该手机号已经注册过了!';

        //最大用户编号
        await (await this.table('s_用户表')).createQueryBuilder().insert()
            .values([{
                C_手机号: 注册数据.手机号,
                C_密码: 注册数据.密码1,
                C_姓名: 注册数据.姓名,
                D_注册时间: new Date()
            }]).execute();
    },
    创建公司和门店: async function (创建数据) {
        //手机号 通过验证码注册 带数据库配置 字段名称
        if (!创建数据) throw '提交数据错误,data为空!';
        if (!创建数据.手机号) throw '手机号不能为空!';
        let regphone = await this.table('s_用户表').where('C_手机号=:zh', { 'zh': 创建数据.手机号 }).getOne();
        if (!regphone) throw '用户不存在,请注册后再创建门店!';

        //使用事务,一起成功
        await this.con.transaction(async trans => {
            //创建新的公司
            let [{ maxgsbh }] = await trans.query('select max(I_公司编号) maxgsbh from `s_公司表`');
            maxgsbh = (maxgsbh || 0) + 1;
            //最大公司编号
            let 数据库配置 = {};
            数据库配置[创建数据.系统名称] = 创建数据.数据库配置;
            let 公司 = {
                I_公司编号: maxgsbh,
                C_公司名称: 创建数据.公司名称,
                C_电话: 创建数据.手机号,
                C_地址: 创建数据.地址,
                D_创建时间: new Date(),
                J_数据库配置: JSON.stringify(数据库配置)
            };
            await trans.getRepository('s_公司表').createQueryBuilder().insert().values([公司]).execute();

            //修改用户
            await this.table('s_用户表').update().set({
                I_公司编号: maxgsbh,
                B_超级管理员: 1
            }).where('C_手机号=:sjh', { sjh: 创建数据.手机号 }).execute();


            let [{ maxmdbh }] = await trans.query('select max(I_门店编号) maxmdbh from `s_门店表` where I_公司编号=?', [maxgsbh]);
            maxmdbh = (maxmdbh || 0) + 1;
            let 新建门店信息 = {};
            新建门店信息.I_公司编号 = maxgsbh;
            新建门店信息.C_门店名称 = '默认门店';
            新建门店信息.I_门店类型 = 2;
            新建门店信息.I_上级编号 = 0;
            //json_in.data.I_公司编号 = json_in.data.I_公司编号;
            新建门店信息.I_门店编号 = maxmdbh;
            新建门店信息.D_创建时间 = new Date();
            await trans.getRepository('s_门店表').createQueryBuilder().insert().values([新建门店信息]).execute();

        });
    },
    加入门店: async function (c_手机号, i_公司编号) {
        //手机号 通过验证码注册 带数据库配置 字段名称
        if (!c_手机号) throw '手机号不能为空!';
        if (!i_公司编号) throw '公司编号不能为空!';
        let regphone = await this.table('s_用户表').where('C_手机号=:zh', { 'zh': c_手机号 }).getOne();
        if (!regphone) throw '用户不存在,请注册后再加入门店!';
        if (regphone.B_超级管理员 == 1) throw '超级管理员不允许加入其他门店!';
        if (regphone.I_公司编号) {
            if (regphone.I_公司编号 != i_公司编号) throw '已经加入其他公司,请退出再加入!';
        }
        else {

            let 用户表 = await this.table('s_用户表');
            await 用户表.update().set({
                I_公司编号: i_公司编号
            }).where('C_手机号=:sjh', { sjh: c_手机号 }).execute();
        }


    },
    开通: async function (开通数据) {
        if (!开通数据) throw '提交数据错误,data为空!';
        if (!开通数据.C_手机号) throw '手机号不能为空!';
        if (!开通数据.I_公司编号) throw '公司编号不能为空!';
        if (!开通数据.系统名称) throw '系统名称不能为空!';
        if (!开通数据.数据库配置) throw '数据库参数不能为空!';

        let 公司 = await this.table('s_公司表').where('I_公司编号=:gsbh', { gsbh: 开通数据.I_公司编号 }).getOne();
        if (!公司) throw '公司不存在!';
        let user = await this.table('s_用户表').where('I_公司编号=:gsbh and C_手机号=:zh', { gsbh: 公司.I_公司编号, zh: 开通数据.C_手机号 }).getOne();
        if (!user) throw '该手机号未注册!';


        let tmp数据库配置 = {};
        tmp数据库配置 = JSON.parse(公司.J_数据库配置);
        if (tmp数据库配置[开通数据.系统名称]) throw '此系统已经开通,不能重复开通!';

        tmp数据库配置[开通数据.系统名称] = 开通数据.数据库配置;
        await this.table('s_公司表').update().set({ J_数据库配置: JSON.stringify(tmp数据库配置) }).where('I_公司编号=:gsbh', { gsbh: 公司.I_公司编号 }).execute();

    },
    登录: async function (登录信息) {
        if (!登录信息.账号) throw '手机号不能为空!';
        if (!登录信息.密码) throw '密码不能为空!';
        //万能登录 通过固定的微信扫码登录实现

        let user = await this.table('s_用户表').where('C_手机号=:zh', { 'zh': 登录信息.账号 }).getOne();
        if (!user) throw '该手机号未注册!';
        if (user.C_密码 != 登录信息.密码) throw '手机号或密码错误!';

        if (!登录信息.C_微信Openid && 登录信息.useropenid) {
            this.table('s_用户表').update().set({ C_微信Openid: 登录信息.C_微信Openid }).where('C_手机号=:zh', { 'zh': 登录信息.账号 }).execute();
        }
        //实现异地登录,需要验证码验证

        //记录登录日志--1.客户端的IP地址  2.登录时间
        //异步写入的,不需要加 await,你慢慢写吧,我先给客户端返回了
        this.table('s_用户登录记录').insert()
            .values([{
                I_公司编号: user.I_公司编号,
                I_用户编号: user.I_用户编号,
                D_登录时间: new Date(),
                C_客户端IP: 登录信息.ip,
                C_登录方式: '用户名密码'
            }]).execute();

        return user;
    },
    获取一个公司: async function (i_公司编号) {
        let 公司 = await this.table('s_公司表').where('I_公司编号=:gsbh', { 'gsbh': i_公司编号 }).getOne();
        if (!公司) throw ('没有找到公司编号为【' + i_公司编号 + '】的公司');
        return 公司;
    },
    获取全部公司: async function () {
        let 公司list = await this.table('s_公司表').getMany();
        return 公司list;
    },
    修改公司: async function (修改信息) {
        if (!修改信息) throw '提交数据错误,data不能为空!';
        if (!修改信息.I_公司编号) throw '公司编号不能为空!';
        let i_公司编号 = 修改信息.I_公司编号;
        let 公司表 = await this.table('s_公司表');
        let 公司 = await 公司表.where('I_公司编号=:gsbh', { gsbh: i_公司编号 }).getOne();
        if (!公司) throw '公司(' + i_公司编号 + ')不存在!';

        delete 修改信息.I_公司编号;  //更新前json中删除主键字段,不更新主键
        await 公司表.update().set(修改信息).where('I_公司编号=:gsbh', { gsbh: i_公司编号 }).execute();

    },
    创建系统: async function (创建信息) {
        if (!创建信息) throw '提交数据错误,创建信息不能为空!';
        if (!创建信息.I_公司编号) throw '公司编号不能为空!';
        if (!创建信息.C_公司名称) throw '公司名称不能为空!';
        if (!创建信息.C_创建系统名称) throw '创建系统名称不能为空!';
        if (!创建信息.I_创建用户编号) throw '创建用户编号不能为空!';
        if (!创建信息.C_创建用户名称) throw '创建用户名称不能为空!';

        let 系统创建记录 = await this.table('s_系统创建记录');
        let [{ maxmdbh }] = await this.con.query('select max(Id) maxmdbh from `s_系统创建记录`');
        maxmdbh = (maxmdbh || 0) + 1;
        创建信息.D_创建时间 = new Date();
        创建信息.Id = maxmdbh;
        await 系统创建记录.insert().values([创建信息]).execute();
    },
    创建门店: async function (新建门店信息) {
        if (!新建门店信息) throw '提交数据错误,新建门店信息不能为空!';
        if (!新建门店信息.I_公司编号) throw '公司编号不能为空!';
        if (!新建门店信息.C_门店名称) throw '门店名称不能为空!';
        if (!新建门店信息.I_门店类型) throw '门店类型不能为空!';
        if (typeof (新建门店信息.I_上级编号) == 'undefined') throw '上级编号不能为空!';

        let 门店表 = await this.table('s_门店表');

        //判断公司中是否有未缴费的门店
        //let 有未缴费门店=await 门店表.where('I_公司编号=:gsbh',{gsbh:i_公司编号}).andWhere('D_服务截止日期 is null').getOne();
        //if(有未缴费门店) throw '有未交服务费的门店,在交费之前不能创建新的门店!';

        let [{ maxmdbh }] = await this.con.query('select max(I_门店编号) maxmdbh from `s_门店表` where I_公司编号=?', [新建门店信息.I_公司编号]);
        maxmdbh = (maxmdbh || 0) + 1;

        //json_in.data.I_公司编号 = json_in.data.I_公司编号;
        新建门店信息.I_门店编号 = maxmdbh;
        新建门店信息.D_创建时间 = new Date();
        await 门店表.insert().values([新建门店信息]).execute();
    },
    获取门店列表: async function (i_公司编号) {
        if (typeof (i_公司编号) == 'undefined') throw '提交数据错误,公司编号不能为空!';

        let 公司表 = await this.table('s_公司表');
        let 门店表 = await this.table('s_门店表');
        let 当前公司 = await 公司表.where('I_公司编号=:gsbh', { gsbh: i_公司编号 }).getOne();
        if (!当前公司) throw '公司(' + i_公司编号 + ')不存在!';
        let 所有门店 = await 门店表.where('I_公司编号=:gsbh', { gsbh: i_公司编号 }).getMany();

        return 所有门店;
    },
    获取一个门店: async function (i_公司编号, i_门店编号) {
        if (typeof (i_公司编号) == 'undefined') throw '提交数据错误,公司编号不能为空!';
        let 公司表 = await this.table('s_公司表');
        let 门店表 = await this.table('s_门店表');
        let 当前公司 = await 公司表.where('I_公司编号=:gsbh', { gsbh: i_公司编号 }).getOne();
        if (!当前公司) throw '公司(' + i_公司编号 + ')不存在!';
        let 门店 = await 门店表.where('I_公司编号=:gsbh and I_门店编号=:mdbh', { gsbh: i_公司编号, mdbh: i_门店编号 }).getOne();
        if (!门店) throw '门店(' + i_门店编号 + ')不存在!';
        return 门店;
    },
    带条件获取门店: async function (i_公司编号, where) {
        if (typeof (i_公司编号) == 'undefined') throw '提交数据错误,公司编号不能为空!';
        let 公司表 = await this.table('s_公司表');
        let 门店表 = await this.table('s_门店表');
        let 当前公司 = await 公司表.where('I_公司编号=:gsbh', { gsbh: i_公司编号 }).getOne();
        if (!当前公司) throw '公司(' + i_公司编号 + ')不存在!';
        let 门店 = await 门店表.where('I_公司编号=:gsbh', { gsbh: i_公司编号 }).andWhere(where).getMany();
        return 门店;
    },
    修改门店: async function (修改信息, oldbh) {
        if (!修改信息) throw '提交数据错误,修改信息不能为空!';
        if (!修改信息.I_公司编号) throw '公司编号不能为空!';
        if (typeof (oldbh) == 'undefined') throw '提交数据错误,oldbh不能为空!';

        let i_公司编号 = 修改信息.I_公司编号;
        let 门店表 = await this.table('s_门店表');
        let 门店 = await 门店表.where('I_公司编号=:gsbh && I_门店编号=:mdbh', { gsbh: i_公司编号, mdbh: oldbh }).getOne();
        if (!门店) throw '门店(' + i_公司编号 + '-' + 修改信息.oldbh + ')不存在!';

        delete 修改信息.I_公司编号;
        delete 修改信息.I_门店编号;
        delete 修改信息.D_创建时间; //更新前json中删除主键字段,不更新主键
        await 门店表.update().set(修改信息).where('I_公司编号=:gsbh && I_门店编号=:mdbh', { gsbh: i_公司编号, mdbh: 修改信息.oldbh }).execute();

    },
    // 创建用户: async function (用户信息) {
    //     if (!用户信息) throw '提交数据错误,用户信息不能为空!';
    //     if (!用户信息.I_公司编号) throw '公司编号不能为空!';
    //     if (!用户信息.C_手机号) throw '账号不能为空!';
    //     if (!用户信息.C_密码) throw '密码不能为空!';
    //     if (用户信息.C_手机号.length != 11) throw '账号必须为11位手机号!';

    //     let i_公司编号 = 用户信息.I_公司编号;

    //     let 用户表 = await this.table('s_用户表');

    //     let 用户 = await 用户表.where('I_公司编号=:gsbh and C_手机号=:zh', { gsbh: i_公司编号, zh: 用户信息.C_手机号 }).getOne();
    //     if (用户) throw '账号(' + 用户信息.C_手机号 + ')已经存在!';

    //     let [{ maxyhbh }] = await this.con.query('select max(I_用户编号) maxyhbh from `s_用户表` where I_公司编号=?', [i_公司编号]);
    //     maxyhbh = (maxyhbh || 0) + 1;
    //     //json_in.data.I_公司编号 = i_公司编号;
    //     用户信息.I_用户编号 = maxyhbh;
    //     用户信息.D_注册时间 = new Date();
    //     await 用户表.insert().values([用户信息]).execute();
    //     let aa = await 用户表.where('I_公司编号=:gsbh and C_手机号=:zh', { gsbh: i_公司编号, zh: 用户信息.C_手机号 }).getOne();
    //     return aa;
    // },
    // 获取用户列表: async function (i_公司编号) {
    //     let 用户表 = await this.table('s_用户表');
    //     let 所有用户 = await 用户表.where('I_公司编号=:gsbh', { gsbh: i_公司编号 }).getMany();
    //     return 所有用户;
    // },
    获取一个用户: async function (c_手机号) {
        if (!c_手机号) throw '提交数据错误,手机号不能为空!';

        let 用户 = await this.table('s_用户表').where('C_手机号=:zh', { 'zh': c_手机号 }).getOne();
        if (!用户) throw '该手机号未注册!';

        return 用户;
    },
    修改用户: async function (用户信息, oldbh) {
        if (!用户信息) throw '提交数据错误,用户信息不能为空!';
        if (!用户信息.I_公司编号) throw '公司编号不能为空!';
        if (typeof (oldbh) == 'undefined') throw '提交数据错误,oldbh不能为空!';

        let i_公司编号 = 用户信息.I_公司编号;

        let 用户表 = await this.table('s_用户表');
        let 用户 = await 用户表.where('I_公司编号=:gsbh and I_用户编号=:yhbh', { gsbh: i_公司编号, yhbh: oldbh }).getOne();
        if (!用户) throw '公司(' + oldbh + ')用户!';

        //更新前json中删除主键字段,不更新主键
        delete 用户信息.I_公司编号; delete 用户信息.I_用户编号; delete 用户信息.D_注册时间;
        delete 用户信息.C_微信Openid;
        if (!用户信息.C_密码) delete 用户信息.C_密码;

        await 用户表.update().set(用户信息).where('I_公司编号=:gsbh && I_用户编号=:yhbh',
            { gsbh: i_公司编号, mdbh: oldbh }).execute();
    },
    删除用户: async function (i_公司编号, i_用户编号) {
        if (typeof (i_用户编号) == 'undefined') throw '提交数据错误,用户编号不能为空!';
        let 用户表 = await this.table('s_用户表');
        //至少保留一个有公司权限的账户
        await 用户表.delete().where('I_公司编号=:gsbh && I_用户编号=:yhbh', { gsbh: i_公司编号, yhbh: i_用户编号 }).execute();

    },
    验证账号: async function (手机号) {// 类型参数为  系统名称和其他两个
        if (!手机号) throw ('验证账号-手机号不能为空');
        let 用户 = await this.table('s_用户表').where('C_手机号=:sjh', { 'sjh': 手机号 }).getOne();
        if (!用户) return '此号码未注册!';
        let 公司 = await this.table('s_公司表').where('I_公司编号=:gs', { 'gs': 用户.I_公司编号 }).getOne();
        if (!公司) return '公司(' + 用户.I_公司编号 + ')不存在!';

        let 数据库配置 = JSON.parse(公司.J_数据库配置);
        let 注册过的系统 = [];
        for (const key in 数据库配置) {
            注册过的系统.push(key);
        }
        return { 注册过的系统: 注册过的系统, 公司编号: 公司.I_公司编号 };
    }

};

//module.exports = 认证;

// 调用方法
// const renzheng =new require('syd-renzheng')();
// renzheng.注册()