const koaRoute = require('koa-router');
const fs = require('fs');
const path = require('path');
/*
    API 提交格式  {cmd:'',}
        返回格式  {cmd:'',result:'成功/失败',data:'', error:'', stack:''}
    */
const baseApi = function (apifun, apiname) {
    return async function (ctx) {
        const start = new Date();
        const json_out = {};
        try {
            if (ctx.req.method != 'POST') throw new Error('只能接受post方法');
            //let str_post = ctx.request.body;
            let json_in = ctx.request.body;
            if (!json_in.cmd) throw new Error('提交数据错误,cmd不能为空');
            global.app.apilogger.info('<p>', apiname, '>>>>', json_in);  //日志
            json_out.cmd = json_in.cmd;
            await apifun(json_in, json_out, ctx);  //异步调用子api方法
            json_out.result = '成功';
        } catch (error) {
            json_out.result = '错误';
            json_out.error = error.message || error;
            if (error.stack) {
                json_out.stack = error.stack;
                global.app.apilogger.error(apiname, 'XXXX', error);
            }
        }
        finally {
            json_out.time = new Date() - start;
            ctx.body = json_out;
            global.app.apilogger.info(apiname, '<<<<', json_out.time, json_out, '</p>');
        }
    };
};

//记录对应的api和路径
let apilist = [];
const ApiAutoLoad = async function (filepath, RootRouter) {
    try {
        let fullpath = path.join(__dirname, '../api/' + filepath);
        let files = await fs.readdirSync(fullpath);
        files.forEach(async file => {
            try {
                let fsstat = await fs.statSync(fullpath + file);
                if (fsstat.isFile()) {
                    if (file.indexOf('.js') > -1) {
                        let filename = file.replace('.js', '');

                        let ApiRoute = new koaRoute();
                        ApiRoute.post('/' + filepath + filename, baseApi(require('../api/' + filepath + filename), '/api/' + filepath + filename));
                        RootRouter.use('/api', ApiRoute.routes(), ApiRoute.allowedMethods());

                        apilist.push({ ApiUrl: '/api/' + filepath + filename, ApiFile: '/api/' + filepath + file });
                    }
                }
                if (fsstat.isDirectory()) {
                    ApiAutoLoad(filepath + file + '/', RootRouter);
                }
            } catch (error) {
                global.app.apilogger.error('加载api文件出错:', error);
            }
        });
    } catch (error) {
        global.app.apilogger.error('加载api文件夹出错:', error);
    }
};

module.exports = function () {
    let RootRouter = new koaRoute();

    //get /api 获取api列表
    let ApiGetRoute = new koaRoute();
    ApiGetRoute.get('/', async function (ctx) {
        let bodystr = '<table><tbody>';
        apilist.forEach(api => { bodystr = bodystr + '<tr><td>' + api.ApiUrl + '</td><td>' + api.ApiFile + '</td></tr>'; });
        ctx.body = bodystr + '</tbody></table>';
    });
    RootRouter.use('/api', ApiGetRoute.routes(), ApiGetRoute.allowedMethods());


    //自动装载api文件夹下的所有api,包括子文件夹,url路径与文件路径对应
    ApiAutoLoad('', RootRouter);
    return RootRouter;
};